#include <iostream>
#include "FDS/rt_api.h"
#include "FDS/rt_fields.h"
#include "FDS/Level2Feature.h"

using namespace std;
using namespace FDS;

static void on_message(int tag, const RT_Message *msg, const RT_Record *rec, void *closure)
{
   cout << "Symbol: " << msg->get_key() << endl;

   if (msg->get(FIDS::MSG_TYPE).to_string() == "D")
   {
      cout << "delete message" << endl;
   }
   else
   {
      if (msg->exists(FIDS::BID_INDEX_1))
      {
         cout << "Bid message: " << endl;
         cout << "Bid Price: " << msg->get(FIDS::BID_1).to_string().c_str() << endl;
         cout << "Bid Volume: "<< msg->get(FIDS::BID_VOL_1).to_string().c_str() << endl;
         cout << "Bid Time: " <<  msg->get(FIDS::BID_TIME_1).to_string().c_str() << endl;
         cout << "Bid Index: " << msg->get(FIDS::BID_INDEX_1).to_string().c_str() << endl;
         cout << "Order Code: " << msg->get(FIDS::ORDER_CODE).to_string().c_str() << endl;
         cout << "Market Maker ID: " << msg->get(FIDS::MARKET_MAKER_ID).to_string().c_str() << endl;
      }
      else
      {
         cout << "Ask message" << endl;
         cout << "Ask Price: " << msg->get(FIDS::ASK_1).to_string().c_str() << endl;
         cout << "Ask Volume: " << msg->get(FIDS::ASK_VOL_1).to_string().c_str() << endl;
         cout << "Ask Time: " << msg->get(FIDS::ASK_TIME_1).to_string().c_str() << endl;
         cout << "Ask Index: " << msg->get(FIDS::ASK_INDEX_1).to_string().c_str() << endl;
         cout << "Order Code: " << msg->get(FIDS::ORDER_CODE).to_string().c_str() << endl;
         cout << "Market Maker ID: " << msg->get(FIDS::MARKET_MAKER_ID).to_string().c_str() << endl;
      }
   }
   cout << endl;
}

int main(int argc, char *argv[])
{
   if(argc < 4)
   {
      cerr << "Usage: " << argv[0] << " <user-serial> <hostname> <full path to key-counter file>" << endl;
      return 1;
   }
   string user_serial(argv[1]);
   string host(argv[2]);
   string full_path(argv[3]);

   // Removing the folder location from full path
   size_t sep_loc = full_path.find_last_of("/");
   std::string path = "";
   std::string device_id = "";

   if(sep_loc != std::string::npos)
   {
      path = full_path.substr(0, sep_loc);
      device_id = full_path.substr(sep_loc+1);
   }
   else
   {
      device_id = full_path;
   }

   // Removing .data extension from the key-counter data file
   size_t ext_loc = device_id.find(".data\0");
   if(ext_loc != std::string::npos)
   {
      device_id = device_id.substr(0, ext_loc);
   }

   FDF::set_connection_info(host.c_str(), user_serial.c_str(), device_id.c_str(), NULL, NULL, path.c_str(), false);
   FDF::connect();

   Level2Feature l2 = Level2Feature();
   if (!FDF::is_active(l2))
   {
      bool res = FDF::enable(l2);
      if (!res)
      {
         cout << "Error enabling Level 2 feature" << endl;
         return -1;
      }
   }

   RT_Request req("FDS_C_L2", "AAPL:L2");
   int tag = 0;
   FDF::request(req, on_message, NULL, &tag);

   while (true)
   {
      FDF::dispatch(-1);
   }

  return 0;
}
